use thiserror::Error;

#[derive(Error, Debug)]
pub enum AnotError {
    #[error("IO error: {0}")]
    Io(#[from] std::io::Error),

    #[error("Serialization error: {0}")]
    Serialization(String),
}
